<!DOCTYPE html>
<html>
<head>
  <title>Rucio UI - Login</title>
  <meta name="viewport" content="width=device-width, initial-scale=1.0" />
  <link rel="shortcut icon" href="/media/favicon.ico" type="image/x-icon">
  <link rel="icon" href="/media/favicon.ico" type="image/x-icon">
  <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/normalize/3.0.1/normalize.min.css">
  <link rel="stylesheet" href="/static/rucio.css?version=9fbb2bab0b085dc2065ed24e8e658787">
  <link rel="stylesheet" href="/static/foundation-icons.css">
  <link rel="stylesheet" href="/static/chosen.min.css">
</head>

<body>
  <nav class="top-bar" data-topbar role="navigation">
    <ul class="title-area">
      <li class="name">
        <h1><a href="/">Rucio UI</a></h1>
      </li>
      <li class="toggle-topbar menu-icon"><a href="#"><span>Menu</span></a></li>
    </ul>
  </nav>

  <div id="content" style="margin: 1em;">
    <div class="row">
      <div class="center">
        <h3>No certificate provided. Please authenticate with a certificate registered in Rucio.</h3>
      </div>
    </div>
    <br>
    <div class="row">
      <div class="center">
        <h5><a href="/ui/login">Use username/password login instead?</a></h5>
      </div>
    </div>
  </div>

</body>
</html>
