
'''
This file is automatically generated; Do not edit it. :)
'''
VERSION_INFO = {
    'final': True,
    'version': '1.30.1',
    'branch_nick': 'release-1.30',
    'revision_id': '41540dc411f2d80dd43585733de4f6929c0f9b04',
    'revno': 12320
}
